@@ -4,7 +4,7 @@ import json
 import logging
 from datetime import datetime, timedelta
 from pathlib import Path
-from typing import Any, Dict, List, Optional, cast
+from typing import Any, Dict, Iterable, List, Optional, Tuple, cast
 
 import alembic  # type: ignore
 import alembic.command  # type: ignore
@@ -51,15 +51,21 @@ Base = declarative_base()
 class Server(Base):
     __tablename__ = "servers"
     guild_xid = Column(BigInteger, primary_key=True, nullable=False)
+    cached_name = Column(String(50))
     created_at = Column(DateTime, nullable=False, default=datetime.utcnow)
     prefix = Column(String(10), nullable=False, default="!")
     expire = Column(Integer, nullable=False, server_default=text("30"))  # minutes
     links = Column(String(10), nullable=False, server_default=text("'public'"))
+    motd = Column(String(10), nullable=False, server_default=text("'both'"))
     power_enabled = Column(Boolean, nullable=False, server_default=true())
     tags_enabled = Column(Boolean, nullable=False, server_default=true())
     create_voice = Column(Boolean, nullable=False, server_default=false())
+    smotd = Column(String(255))
     games = relationship("Game", back_populates="server", uselist=True)
     channels = relationship("Channel", back_populates="server", uselist=True)
+    channel_settings = relationship(
+        "ChannelSettings", back_populates="server", uselist=True
+    )
     teams = relationship("Team", back_populates="server", uselist=True)
 
     def bot_allowed_in(self, channel_xid: int) -> bool:
@@ -67,6 +73,25 @@ class Server(Base):
             channel.channel_xid == channel_xid for channel in self.channels
         )
 
+    @classmethod
+    def recent_metrics(cls, session: Session) -> dict:
+        data = [
+            row[1]
+            for row in (
+                session.query(
+                    func.date(Server.created_at).label("day"),
+                    func.count(Server.guild_xid),
+                )
+                .filter(
+                    Server.created_at >= datetime.utcnow() - timedelta(days=5),
+                )
+                .group_by("day")
+                .order_by(desc("day"))
+                .all()
+            )
+        ]
+        return {f"servers_{i}": count for i, count in enumerate(data)}
+
     def games_data(self) -> Dict[str, List[str]]:
         data: Dict[str, List[str]] = {
             "id": [],
@@ -156,6 +181,19 @@ class Channel(Base):
     server = relationship("Server", back_populates="channels")
 
 
+class ChannelSettings(Base):
+    __tablename__ = "channel_settings"
+    channel_xid = Column(BigInteger, primary_key=True, nullable=False)
+    guild_xid = Column(
+        BigInteger,
+        ForeignKey("servers.guild_xid", ondelete="CASCADE"),
+        nullable=False,
+        index=True,
+    )
+    default_size = Column(Integer, nullable=True)
+    server = relationship("Server", back_populates="channel_settings")
+
+
 games_tags = Table(
     "games_tags",
     Base.metadata,
@@ -192,6 +230,25 @@ class User(Base):
     power = Column(Integer, nullable=True)
     game = relationship("Game", back_populates="users")
 
+    @classmethod
+    def recent_metrics(cls, session: Session) -> dict:
+        data = [
+            row[1]
+            for row in (
+                session.query(
+                    func.date(User.created_at).label("day"),
+                    func.count(User.xid),
+                )
+                .filter(
+                    User.created_at >= datetime.utcnow() - timedelta(days=5),
+                )
+                .group_by("day")
+                .order_by(desc("day"))
+                .all()
+            )
+        ]
+        return {f"users_{i}": count for i, count in enumerate(data)}
+
     @property
     def waiting(self) -> bool:
         if self.game and self.game.status in ["pending", "ready"]:
@@ -316,25 +373,49 @@ class Game(Base):
     reports = relationship("Report", back_populates="game", uselist=True)
 
     @classmethod
-    def recent_metrics(cls, session: Session) -> List[int]:
-        return [
-            row[1]
-            for row in (
-                session.query(
-                    func.date(Game.created_at).label("day"),
-                    func.count(Game.id),
-                )
-                .filter(
-                    and_(
-                        Game.status == "started",
-                        Game.created_at >= datetime.utcnow() - timedelta(days=5),
-                    )
-                )
-                .group_by("day")
-                .order_by(desc("day"))
-                .all()
+    def games_per_day(cls, session: Session) -> Iterable[Tuple[datetime, int]]:
+        return cast(
+            Iterable[Tuple[datetime, int]],
+            session.query(
+                func.date(Game.created_at).label("day"),
+                func.count(Game.id),
             )
+            .filter(
+                and_(
+                    Game.status == "started",
+                    Game.created_at >= datetime.utcnow() - timedelta(days=5),
+                )
+            )
+            .group_by("day")
+            .order_by(desc("day"))
+            .all(),
+        )
+
+    @classmethod
+    def games_per_day_per_channel(
+        cls, session: Session, guild_xid: int
+    ) -> Iterable[Tuple[datetime, int, int]]:
+        filters = [
+            Game.status == "started",
+            Game.guild_xid == guild_xid,
         ]
+        return cast(
+            Iterable[Tuple[datetime, int, int]],
+            session.query(
+                func.date(Game.created_at).label("day"),
+                Game.channel_xid,
+                func.count(Game.id),
+            )
+            .filter(and_(*filters))
+            .group_by("day", Game.channel_xid)
+            .order_by(desc("day"), Game.channel_xid)
+            .all(),
+        )
+
+    @classmethod
+    def recent_metrics(cls, session: Session) -> dict:
+        data = [row[1] for row in cls.games_per_day(session)]
+        return {f"games_{i}": count for i, count in enumerate(data)}
 
     @property
     def power(self) -> Optional[float]:
@@ -366,7 +447,7 @@ class Game(Base):
             Game.size == size,
             Game.channel_xid == channel_xid,
             Game.system == system,
-            Game.message_xid != None,
+            Game.message_xid.isnot(None),
         ]
         having_filters = [
             func.count(distinct(games_tags.c.tag_id)) == len(required_tag_ids),
@@ -380,7 +461,7 @@ class Game(Base):
                 else:
                     having_filters.append(func.avg(User.power) < 7)
             else:
-                select_filters.append(User.power == None)
+                select_filters.append(User.power.is_(None))
         inner = (
             session.query(Game.id)
             .join(User, isouter=True)
@@ -433,7 +514,7 @@ class Game(Base):
             session.query(Game)
             .filter(
                 and_(
-                    Game.voice_channel_xid != None,
+                    Game.voice_channel_xid.isnot(None),
                     datetime.utcnow() - timedelta(minutes=10) >= Game.updated_at,
                 )
             )
@@ -467,7 +548,7 @@ class Game(Base):
                 and_(
                     Game.created_at >= datetime.utcnow() - timedelta(hours=1),
                     Game.status == "started",
-                    Game.channel_xid != None,
+                    Game.channel_xid.isnot(None),
                 )
             )
             .group_by(Game.guild_xid, Game.channel_xid)
@@ -500,6 +581,15 @@ class Game(Base):
 
     def to_embed(self, dm: bool = False, wait: Optional[float] = None) -> discord.Embed:
         prefix = self.server.prefix
+        show_motd = (
+            True
+            if self.server.motd == "both"
+            else True
+            if dm and self.server.motd == "private"
+            else True
+            if not dm and self.server.motd == "public"
+            else False
+        )
         show_link = True if dm else self.server.links == "public"
         if self.status == "pending":
             remaining = int(self.size) - len(cast(List[User], self.users))
@@ -526,9 +616,17 @@ class Game(Base):
             )
         elif self.system == "spelltable":
             if show_link:
-                description += (
-                    f"Click the link below to join your SpellTable game.\n<{self.url}>"
-                )
+                if self.url:
+                    description += (
+                        "Click the link below to join your SpellTable game."
+                        f"\n<{self.url}>"
+                    )
+                else:
+                    description += (
+                        "Sorry but SpellBot was unable to create a SpellTable link"
+                        " for this game. Please go to"
+                        " [spelltable.com](https://www.spelltable.com/) to create one.\n"
+                    )
             else:
                 description += (
                     "Please check your Direct Messages for your SpellTable link."
@@ -541,6 +639,8 @@ class Game(Base):
             description += (
                 "Please exchange Arena contact information and head over there to play!"
             )
+        if self.server.smotd and show_motd:
+            description += f"\n\n{self.server.smotd}"
         embed.description = description
 
         if self.voice_channel_xid:
@@ -561,7 +661,7 @@ class Game(Base):
             players = ", ".join(
                 sorted(
                     [
-                        f"<@{user.xid}>"
+                        f"<@{user.xid}>"  # Support <@!USERID> for server nick?
                         + (
                             f" (Power: {user.power})"
                             if (user.power and self.server.power_enabled)
